#ifndef COLOR_H
#define COLOR_H

#include <cmath>

class Color
{
public:
	union
	{
		struct
		{
			float r, g, b, a;
		};
		
		float data[4];
	};
	

	Color()
	{
	}

	Color(float r, float g, float b, float a = 1.0f) : r(r), g(g), b(b), a(a)
	{
	}

	Color(const Color& v) : r(v.r), g(v.g), b(v.b), a(v.a)
	{
	}

	Color& operator=(const Color& v)
	{
		r = v.r;
		g = v.g;
		b = v.b;
		a = v.a;

		return *this;
	}

	Color& operator+=(const Color& v)
	{
		r += v.r;
		g += v.g;
		b += v.b;
		a += v.a;

		return *this;
	}

	Color& operator -=(const Color& v)
	{
		r -= v.r;
		g -= v.g;
		b -= v.b;
		a -= v.a;

		return *this;
	}

	Color& operator *=(float f)
	{
		r *= f;
		g *= f;
		b *= f;
		a *= f;

		return *this;
	}

	Color& operator/=(float f)
	{
		float invF = 1.0f / f;
		
		r *= invF;
		g *= invF;
		b *= invF;
		a *= invF;

		return *this;
	}

	Color operator+(const Color& v) const
	{
		return Color(r + v.r, g + v.g, b + v.b, a + v.a);
	}

	Color operator-(const Color& v) const
	{
		return Color(r - v.r, g - v.g, b - v.b, a - v.a);
	}

	Color operator *(float f) const
	{
		return Color(r * f, g * f, b * f, a * f);
	}

	Color operator/(float f) const
	{
		float invF = 1.0f / f;
		return Color(r * invF, g * invF, b * invF, a * invF);
	}
};

#endif