#ifndef FIRE_EMITER
#define FIRE_EMITER

#include "ParticleEmiter.h"

class FireEmiter : public ParticleEmiter
{
	Vec2D position;
	float halfWidth;
	float totalTime;

	const UVRegion* regions[4];
	int type;

public:
	FireEmiter(float particlesPerSecond, const Vec2D& position, float width, const TextureRef& texture) 
		: ParticleEmiter(particlesPerSecond), position(position), halfWidth(width / 2),
		  totalTime(0), type(0)
	{
		regions[0] = texture->GetNamedRegion("type1");
		regions[1] = texture->GetNamedRegion("type2");
		regions[2] = texture->GetNamedRegion("type3");
		regions[3] = texture->GetNamedRegion("type4");
	}

	bool Update(ParticleSystem* ps, float dt)
	{
		timeAccumulator += dt;
		totalTime += dt;

		const Color colorA(1, 0.184f, 0.02f);
		const Color colorB(1, 0.5f, 0.05f);

		while (timeAccumulator > timePerParticle)
		{
			Particle* particle = ps->CreateParticle();

			Vec2D v = ps->random.GetPointOnCircle(halfWidth);

			particle->position.x = position.x + v.x;
			particle->position.y = position.y + abs(v.y) * 0.6f;

			Color color = ps->random.GetColor(colorA, colorB);

			particle->velocity.x = ps->random.GetFloat(-7, 7) + 7 * sin(5*totalTime);
			particle->velocity.y = -100;
			
			particle->current.color = color;
			particle->current.angle = ps->random.GetAngle();
			particle->current.size = ps->random.GetFloat(8, 40);

			particle->change.color = Color(color.r, color.g, color.b, 0);

			particle->change.angle = particle->current.angle + 0.5f;
			particle->change.size = 0;
			particle->changeDuration = ps->random.GetFloat(0.5f, 3.5f) * (0.2f + 0.8f * abs(v.y) / halfWidth);

			particle->CalculateChangePerSecond();

			particle->uvRegion = regions[type];

			type = (type + 1) % 4;

			timeAccumulator -= timePerParticle;
		}

		return true;
	}
};

#endif