#ifndef PARTICLE_SAMPLE_H
#define PARTICLE_SAMPLE_H

#include "ParticleSystem.h"
#include "SimpleProfiler.h"

extern bool keys[];

#include "SampleEmiter.h"
#include "SnowflakeEmiter.h"
#include "FireEmiter.h"


class ParticleSample
{
	ParticleSystemRef particleSystem;
	RenderingHelper* helper;

	int currentSample;

	ParticleEmiterRef sampleEmiter;
	ParticleEmiterRef fireEmiter;
	ParticleEmiterRef snowflakeEmiter;

	TextureRef particle;
	TextureRef fireParticle;
	TextureRef snowflakeParticle;

	const char* label;
public:
	ParticleSample() : currentSample(-1), label(0)
	{
		helper = RenderingHelper::GetInstance();

		particle = Texture::Load("particle.png");

		snowflakeParticle = Texture::Load("snowflake_particle.png");

		snowflakeParticle->SetNamedRegion("type1", 0.0f, 0.0f, 0.5f, 0.5f);
		snowflakeParticle->SetNamedRegion("type2", 0.5f, 0.0f, 1.0f, 0.5f);
		snowflakeParticle->SetNamedRegion("type3", 0.0f, 0.5f, 0.5f, 1.0f);
		snowflakeParticle->SetNamedRegion("type4", 0.5f, 0.5f, 1.0f, 1.0f);

		fireParticle = Texture::Load("fire_particle.png");

		fireParticle->SetNamedRegion("type1", 0.0f, 0.0f, 0.5f, 0.5f);
		fireParticle->SetNamedRegion("type2", 0.5f, 0.0f, 1.0f, 0.5f);
		fireParticle->SetNamedRegion("type3", 0.0f, 0.5f, 0.5f, 1.0f);
		fireParticle->SetNamedRegion("type4", 0.5f, 0.5f, 1.0f, 1.0f);
		
		particleSystem = ParticleSystemRef(new ParticleSystem());

		sampleEmiter = ParticleEmiterRef(new SampleEmiter(100));
		snowflakeEmiter = ParticleEmiterRef(new SnowflakeEmiter(100, snowflakeParticle));
		fireEmiter = ParticleEmiterRef(new FireEmiter(600, Vec2D(320, 320), 60, fireParticle));
	}

	void Render()
	{
		glClear(GL_COLOR_BUFFER_BIT);

		{
			SimpleProfiler profiler(helper, 4, "Render time");
			particleSystem->Render();
		}

		helper->DrawDebugInformation();

		SDL_GL_SwapBuffers();
	}

	bool Update(float dt)
	{
		helper->WriteDebugLine(0, "Press [1]-[4] to select emiter");

		if (label) 
			helper->WriteDebugLine(1, label);

		int option = -1;

		for (int k = SDLK_1; k <= SDLK_9; k++)
		{
			if (keys[k]) option = k - SDLK_0;
		}

		if (option > 0 && option != currentSample)
		{
			currentSample = option;

			particleSystem->Reset();

			switch (currentSample)
			{
			case 1:
				label = "Sample emiter without texture";
				helper->SetClearColor(Color(0.5f, 0.5f, 0.5f));
				particleSystem->AddEmiter(sampleEmiter);
				particleSystem->SetTexture();
				particleSystem->SetBlendMode(BlendModes::Normal);
				break;
			case 2:
				label = "Sample emiter with texture";
				helper->SetClearColor(Color(0.5f, 0.5f, 0.5f));
				particleSystem->AddEmiter(sampleEmiter);
				particleSystem->SetTexture(particle);
				particleSystem->SetBlendMode(BlendModes::Normal);
				break;
			case 3:
				label = "Snowflake emiter";
				helper->SetClearColor(Color(0, 0, 0));
				particleSystem->AddEmiter(snowflakeEmiter);
				particleSystem->SetTexture(snowflakeParticle);
				particleSystem->SetBlendMode(BlendModes::Normal);
				break;
			case 4:
				label = "Fire emiter";
				helper->SetClearColor(Color(0, 0, 0));
				particleSystem->AddEmiter(fireEmiter);
				particleSystem->SetTexture(fireParticle);
				particleSystem->SetBlendMode(BlendModes::Add);
				break;
			}
		}


		{
			SimpleProfiler profiler(helper, 3, "Update time");
			particleSystem->Update(dt);
		}
		
		helper->WriteDebugLine(5, "particles = %i", particleSystem->GetParticlesCount());

		return true;
	}
};


#endif