#ifndef RANDOM_H
#define RANDOM_H

#include "Color.h"
#include "Vec2D.h"

class Random
{
	int lastRandom;
public:
	Random()
	{
		lastRandom = 42;
	}

	Random(int seed)
	{
		lastRandom = seed;
	}

	void Seed(int seed)
	{
		lastRandom = seed;
	}

	int GetInt()
	{
		return (lastRandom = (lastRandom * 3877 + 29573) % 139968);
	}

	int GetInt(int min, int max)
	{
		return min + GetInt() % (max - min + 1);
	}

	float GetFloat()
	{
		return (lastRandom = (lastRandom * 3877 + 29573) % 139968) / 139967.0f;
	}

	float GetFloat(float min, float max)
	{
		return min + GetFloat() * (max - min);
	}

	Color GetColor()
	{
		return Color(GetFloat(), GetFloat(), GetFloat(), GetFloat());
	}

	Color GetOpaqueColor()
	{
		return Color(GetFloat(), GetFloat(), GetFloat(), 1);
	}

	Color GetTransparentColor()
	{
		return Color(GetFloat(), GetFloat(), GetFloat(), 0);
	}

	Color GetColor(const Color& colorA, const Color& colorB)
	{
		float r = GetFloat(colorA.r, colorB.r);
		float g = GetFloat(colorA.g, colorB.g);
		float b = GetFloat(colorA.b, colorB.b);
		float a = GetFloat(colorA.a, colorB.a);

		return Color(r, g, b, a);
	}

	float GetAngle()
	{
		return GetFloat() * 6.28318531f;
	}

	Vec2D GetPointOnCircle(float r = 1.0f)
	{
		float angle = GetAngle();

		return Vec2D(r * cos(angle), r * sin(angle));
	}

	Vec2D GetVec2D(float minX, float maxX, float minY, float maxY)
	{
		return Vec2D(GetFloat(minX, maxX), GetFloat(minY, maxY));
	}
};

#endif