#ifndef TEXTURE_H
#define TEXTURE_H

#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>
#include <boost/shared_ptr.hpp>

#include <map>
#include <string>

class Texture;

typedef boost::shared_ptr<Texture> TextureRef;

// koordynaty fragmentu tekstury
struct UVRegion
{
	// lewy grny rg
	float u0, v0;

	// prawy dolny rg
	float u1, v1;

	UVRegion()
	{
	}

	UVRegion(float u0, float v0, float u1, float v1) 
		: u0(u0), v0(v0), u1(u1), v1(v1)
	{
	}
};

extern const UVRegion DefaultRegion;

class Texture
{
	GLuint textureId;
	int width;
	int height;

	std::map<std::string, UVRegion*> regions;

	Texture(const char* fileName);
	bool Initialize(SDL_Surface* surface);
public:
	static TextureRef Load(const char* fileName);

	int GetWidth()
	{
		return width;
	}

	int GetHeight()
	{
		return height;
	}

	void SetAsCurrent();

	~Texture();

	const UVRegion* SetNamedRegion(const std::string& name, float u0, float v0, float u1, float v1);
	const UVRegion* GetNamedRegion(const std::string& name);
	void ReleaseNamedRegions();
};



#endif