#ifndef VEC2D_H
#define VEC2D_H

#include <cmath>

class Vec2D
{
public:
	union
	{
		struct
		{
			float x, y;
		};
		
		float data[2];
	};
	

	Vec2D()
	{
	}

	Vec2D(float x, float y) : x(x), y(y)
	{
	}

	Vec2D(const Vec2D& v) : x(v.x), y(v.y)
	{
	}

	Vec2D& operator=(const Vec2D& v)
	{
		x = v.x;
		y = v.y;

		return *this;
	}

	Vec2D& operator+=(const Vec2D& v)
	{
		x += v.x;
		y += v.y;

		return *this;
	}

	Vec2D& operator -=(const Vec2D& v)
	{
		x -= v.x;
		y -= v.y;

		return *this;
	}

	Vec2D& operator *=(const Vec2D& v)
	{
		x *= v.x;
		y *= v.y;

		return *this;
	}

	Vec2D& operator *=(float f)
	{
		x *= f;
		y *= f;

		return *this;
	}

	Vec2D& operator/=(float f)
	{
		float invF = 1.0f / f;
		
		x *= invF;
		y *= invF;

		return *this;
	}

	Vec2D operator+(const Vec2D& v) const
	{
		return Vec2D(x + v.x, y + v.y);
	}

	Vec2D operator-(const Vec2D& v) const
	{
		return Vec2D(x - v.x, y - v.y);
	}

	Vec2D operator*(const Vec2D& v) const
	{
		return Vec2D(x * v.x, y * v.y);
	}

	Vec2D operator*(float f) const
	{
		return Vec2D(x * f, y * f);
	}

	Vec2D operator/(float f) const
	{
		float invF = 1.0f / f;
		return Vec2D(x * invF, y * invF);
	}

	float Length() const
	{
		return sqrt(x * x + y * y);
	}

	void Normalize()
	{
		float invLen = 1.0f / Length();

		x *= invLen;
		y *= invLen;
	}

	Vec2D Normalized() const
	{
		float invLen = 1.0f / Length();

		return Vec2D(x * invLen, y * invLen);
	}
};

#endif