#include <iostream>
using namespace std;

#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>

#include "ParticleSample.h"
#include "Texture.h"

#define SCREEN_WIDTH  640
#define SCREEN_HEIGHT 480
#define FRAME_TIME    (1000.0f / 30)

bool InitOGL()
{
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	glEnable(GL_TEXTURE_2D);

	glClearColor(0, 0, 0, 1.0f);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, SCREEN_WIDTH, SCREEN_HEIGHT, 0, -1, 1);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	return (glGetError() == GL_NO_ERROR);
}

bool keys[SDLK_LAST];

int main(int argc, char* argv[])
{
	if (SDL_Init(SDL_INIT_EVERYTHING) < 0) return -1;

    if (SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, 32, SDL_OPENGL) == NULL) return -1;
    
	if (!InitOGL()) return -1;

	SDL_WM_SetCaption("ParticleSystem v1", NULL);

	bool quit = false;

	SDL_Event sdlEvent;

	ParticleSample sample;

	Uint32 previousTime = SDL_GetTicks();
	float timeAcumulator = 0;

	while (!quit)
	{
		while (SDL_PollEvent(&sdlEvent))
		{
			if (sdlEvent.type == SDL_QUIT) quit = true;

			if (sdlEvent.type == SDL_KEYDOWN)
				keys[sdlEvent.key.keysym.sym] = true;

			if (sdlEvent.type == SDL_KEYUP)
				keys[sdlEvent.key.keysym.sym] = false;
		}

		Uint32 currentTime = SDL_GetTicks();

		timeAcumulator += currentTime - previousTime;

		if (timeAcumulator >= FRAME_TIME * 5)
		{
			// zbyt dua przerwa, zmniejszenie
			timeAcumulator = FRAME_TIME;
		}

		while (timeAcumulator >= FRAME_TIME)
		{
			float dt = FRAME_TIME / 1000.0f;

			if (sample.Update(dt))
			{
				sample.Render();
			}
			else
			{
				quit = true;
				break;
			}

			timeAcumulator -= FRAME_TIME;
		}

        previousTime = currentTime;

        SDL_Delay(1);   
	}

	return 0;
}